# Generated by roxygen2: do not edit by hand

S3method(logLik,osm)
S3method(model.frame,osm)
S3method(nobs,osm)
S3method(print,clustord)
S3method(print,osm)
S3method(print,summary.clustord)
S3method(print,summary.osm)
S3method(reorder,clustord)
S3method(summary,clustord)
S3method(summary,osm)
S3method(vcov,osm)
export(calc_SE_bicluster)
export(calc_SE_rowcluster)
export(calc_cluster_comparisons)
export(clustord)
export(mat_to_df)
export(osm)
export(rerun)
importFrom(Rcpp,evalCpp)
importFrom(flexclust,randIndex)
importFrom(methods,is)
importFrom(stats,.getXlevels)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,deviance)
importFrom(stats,formula)
importFrom(stats,glm)
importFrom(stats,glm.fit)
importFrom(stats,kmeans)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,model.response)
importFrom(stats,model.weights)
importFrom(stats,naprint)
importFrom(stats,nobs)
importFrom(stats,optim)
importFrom(stats,optimHess)
importFrom(stats,pnorm)
importFrom(stats,reorder)
importFrom(stats,runif)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,vcov)
importFrom(utils,tail)
useDynLib(clustord, .registration=TRUE)
