## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(echo = FALSE)

## ----message=FALSE,warning=FALSE----------------------------------------------
require(knitr)
require(formatR)
# Set so that long lines in R will be wrapped:
opts_chunk$set(tidy.opts=list(width.cutoff=40),tidy=TRUE)

## ----out.width="70%", fig.align="center"--------------------------------------
knitr::include_graphics('vignette_fig/survey_table1.png')

